from django.db import models
from rest_framework import serializers
from rest_framework.filters import BaseFilterBackend

from ..helpers import get_annotations


class ColumnIndexFilter(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        params = request.query_params

        column_index = params.get("columnIndex", None)
        if not column_index:
            return queryset

        aggregation = params.get("aggregation", None)
        if not aggregation:
            raise serializers.ValidationError({"error": "Aggregation is mandatory."})

        aggregation_field = params["aggregationField"].replace(".", "__") if "aggregationField" in params else None

        annotations = get_annotations(
            aggregation=aggregation,
            aggregation_field=aggregation_field,
            percentile=params.get("percentile", None),
            additional_filter=params.get("additionalFilter", None),
            queryset=queryset,
        )
        indexes = {}
        for column in column_index.split(","):
            index_column = f'{column}__index'
            indexes[index_column] = self._get_sorting_annotation(
                queryset=queryset,
                field_name=column,
                annotations=annotations)

        queryset = queryset.annotate(**indexes)

        return queryset

    @staticmethod
    def _get_sorting_annotation(queryset: models.QuerySet,
                                field_name: str,
                                annotations: dict) -> models.Case:
        queryset = queryset.values(field_name)
        queryset = queryset.annotate(**annotations)
        queryset = queryset.order_by('value')

        return models.Case(*[
            models.When(**{
                "{}".format(field_name): group[field_name],
                "then": index
            }) for index, group in enumerate(queryset)
        ], output_field=models.IntegerField())



from datetime import date, datetime, timedelta

RECORDS = [
    {"group1": "2",
     "group2": "1",
     "integer": 0,
     "float": 0,
     "date": date(2020, 11, 2),
     "datetime": datetime(2020, 10, 1, 0, 1),
     "duration": timedelta(days=1)},

    {"group1": "1",
     "group2": "2",
     "integer": 1,
     "float": 1,
     "date": date(2020, 10, 1),
     "datetime": datetime(2020, 11, 1, 0, 5),
     "duration": timedelta(days=2)},

    {"group1": "1",
     "group2": "1",
     "integer": 2,
     "float": 2,
     "date": date(2020, 11, 1),
     "datetime": datetime(2020, 10, 31, 0, 3),
     "duration": timedelta(days=3)},

    {"group1": "3",
     "group2": "2",
     "integer": 3,
     "float": 3,
     "date": date(2020, 10, 31),
     "datetime": datetime(2020, 11, 1, 0, 4),
     "duration": timedelta(days=4)},

    {"group1": "3",
     "group2": "1",
     "integer": 4,
     "float": 4,
     "date": date(2020, 11, 1),
     "datetime": datetime(2020, 10, 1, 0, 2),
     "duration": timedelta(days=5)},

    {"group1": "3",
     "group2": "3",
     "integer": 5,
     "float": 5,
     "date": date(2020, 10, 1),
     "datetime": datetime(2020, 11, 2, 0, 6),
     "duration": timedelta(days=6)},
]



from django.db import connection
from rest_framework.test import APITestCase
from parameterized import parameterized
from .models import TestCaseModel
from .fixtures import RECORDS
from .test_data import ANNOTATIONS_TESTING
from .test_data import SORTED_GROUPS_TESTING
from .test_data import UNSORTED_GROUPS_TESTING


class AggregationTests(APITestCase):
    URL = "/test/aggregation"

    def setUp(self):
        for record in RECORDS:
            record = TestCaseModel(**record)
            record.save()

    @parameterized.expand(ANNOTATIONS_TESTING)
    def test_annotations(self, query, expected_response):
        if connection.vendor != "postgresql" and query["aggregation"] == "percentile":
            self.skipTest("Percentile only works with PostgreSQL")
        response = self.client.get(self.URL, query, format="json")
        self.assertEqual(response.status_code, 200,
                         msg=f"Failed on: {query}"
                             f"\nResponse: {response.data}")
        self.assertEqual(response.data, expected_response,
                         msg=f"Failed on: {query}"
                             f"\nResponse: {response.data}"
                             f"\nExpected: {expected_response}")

    @parameterized.expand(UNSORTED_GROUPS_TESTING)
    def test_group_by(self, query, expected_response):
        response = self.client.get(self.URL, query, format="json")
        self.assertEqual(response.status_code, 200,
                         msg=f"Failed on: {query}"
                             f"\nResponse: {response.data}")
        self.assertEqual(len(response.data), len(expected_response),
                         msg=f"Failed on: {query}"
                             f"\nResponse: {response.data}"
                             f"\nExpected: {expected_response}")
        for result in response.data:
            self.assertIn(result, expected_response,
                          msg=f"Failed on: {query}"
                              f"\nResponse: {response.data}"
                              f"\nExpected: {expected_response}")

    @parameterized.expand(SORTED_GROUPS_TESTING)
    def test_group_by_fields(self, query, expected_response):
        response = self.client.get(self.URL, query, format="json")
        self.assertEqual(response.status_code, 200,
                         msg=f"Failed on: {query}"
                             f"\nResponse: {response.data}")
        self.assertEqual(len(response.data), len(expected_response),
                         msg=f"Failed on: {query}"
                             f"\nResponse: {response.data}"
                             f"\nExpected: {expected_response}")
        self.assertListEqual(response.data, expected_response,
                             msg=f"Failed on: {query}"
                                 f"\nResponse: {response.data}"
                                 f"\nExpected: {expected_response}")



import json
from datetime import date, datetime, timedelta

ANNOTATIONS_TESTING = [
    ({"aggregation": "count"},
     {"value": 6}),

    ({"aggregation": "sum", "aggregationField": "integer"},
     {"value": 15}),

    ({"aggregation": "average", "aggregationField": "integer"},
     {"value": 2.5}),

    ({"aggregation": "minimum", "aggregationField": "integer"},
     {"value": 0}),

    ({"aggregation": "maximum", "aggregationField": "integer"},
     {"value": 5}),

    ({"aggregation": "percentile", "aggregationField": "integer",
      "percentile": 0.5},
     {"value": 2.5}),

    ({"aggregation": "sum", "aggregationField": "float"},
     {"value": 15}),

    ({"aggregation": "average", "aggregationField": "float"},
     {"value": 2.5}),

    ({"aggregation": "minimum", "aggregationField": "float"},
     {"value": 0}),

    ({"aggregation": "maximum", "aggregationField": "float"},
     {"value": 5}),

    ({"aggregation": "percentile", "aggregationField": "float",
      "percentile": 0.5},
     {"value": 2.5}),

    ({"aggregation": "minimum", "aggregationField": "date"},
     {"value": date(2020, 10, 1)}),

    ({"aggregation": "maximum", "aggregationField": "date"},
     {"value": date(2020, 11, 2)}),

    ({"aggregation": "minimum", "aggregationField": "datetime"},
     {"value": datetime(2020, 10, 1, 0, 1)}),

    ({"aggregation": "maximum", "aggregationField": "datetime"},
     {"value": datetime(2020, 11, 2, 0, 6)}),

    ({"aggregation": "sum", "aggregationField": "duration"},
     {"value": timedelta(days=21)}),

    ({"aggregation": "average", "aggregationField": "duration"},
     {"value": timedelta(days=3, hours=12)}),

    ({"aggregation": "minimum", "aggregationField": "duration"},
     {"value": timedelta(days=1)}),

    ({"aggregation": "maximum", "aggregationField": "duration"},
     {"value": timedelta(days=6)}),

    ({"aggregation": "percentile", "aggregationField": "duration",
      "percentile": 0.5},
     {"value": timedelta(days=3, hours=12)}),

    ({"aggregation": "percent",
      "additionalFilter": json.dumps({"type": "operator",
                                      "data": {"attribute": "group2",
                                               "operator": "=",
                                               "value": "1"}})},
     {"numerator": 3, "denominator": 6, "value": 0.5}),
]

UNSORTED_GROUPS_TESTING = [
    # GROUP BY ONE FIELD
    ({"aggregation": "count", "groupBy": "group1"},
     [{"group1": "1", "value": 2},
      {"group1": "2", "value": 1},
      {"group1": "3", "value": 3}]),

    # GROUP BY MULTIPLE FIELDS
    ({"aggregation": "count", "groupBy": "group1,group2"},
     [{"group1": "3", "group2": "3", "value": 1},
      {"group1": "3", "group2": "2", "value": 1},
      {"group1": "1", "group2": "2", "value": 1},
      {"group1": "2", "group2": "1", "value": 1},
      {"group1": "3", "group2": "1", "value": 1},
      {"group1": "1", "group2": "1", "value": 1},
      ]),

    # IGNORE LIMIT BY AND SHOW OTHER WITHOUT LIMIT
    ({"aggregation": "count", "groupBy": "group1",
      "limitBy": "group1", "showOther": 1},
     [{"group1": "1", "value": 2},
      {"group1": "2", "value": 1},
      {"group1": "3", "value": 3}]),
]

SORTED_GROUPS_TESTING = [
    # DESCENDING SORTING BY VALUE
    ({"aggregation": "count", "groupBy": "group1", "orderBy": "-value"},
     [{"group1": "3", "value": 3},
      {"group1": "1", "value": 2},
      {"group1": "2", "value": 1}]),

    # ASCENDING SORTING BY VALUE
    ({"aggregation": "count", "groupBy": "group1", "orderBy": "value"},
     [{"group1": "2", "value": 1},
      {"group1": "1", "value": 2},
      {"group1": "3", "value": 3}]),

    # SORT BY GROUP NAME
    ({"aggregation": "count", "groupBy": "group1", "orderBy": "group1"},
     [{"group1": "1", "value": 2},
      {"group1": "2", "value": 1},
      {"group1": "3", "value": 3}]),

    # GROUP BY MULTIPLE FIELDS WITH SORT BY TOTAL GROUP VALUE
    ({"columnIndex": "group1",
      "aggregation": "count",
      "groupBy": "group1,group2", "orderBy": "-group1__index,-group2"},
     [{"group1": "3", "group2": "3", "value": 1},
      {"group1": "3", "group2": "2", "value": 1},
      {"group1": "3", "group2": "1", "value": 1},
      {"group1": "1", "group2": "2", "value": 1},
      {"group1": "1", "group2": "1", "value": 1},
      {"group1": "2", "group2": "1", "value": 1},
      ]),

    # LIMIT NUMBER OF RETURNED GROUPS
    ({"aggregation": "count", "groupBy": "group1", "orderBy": "-value",
      "limit": 1},
     [{'group1': '3', 'value': 3}]),

    # LIMIT WITH GROUP BY MULTIPLE FIELDS
    ({"columnIndex": "group1",
      "aggregation": "count",
      "groupBy": "group1,group2", "orderBy": "-group1__index,-group2",
      "limit": 1},
     [{'group1': '3', 'group2': '3', 'value': 1},
      {'group1': '3', 'group2': '2', 'value': 1},
      {'group1': '3', 'group2': '1', 'value': 1}]),

    # LIMIT BY NOT FIRST FIELD IN GROUP BY
    ({"columnIndex": "group2",
      "aggregation": "count",
      "groupBy": "group1,group2", "orderBy": "group2__index",
      "limit": 1, "limitBy": "group2"},
     [{'group1': '3', 'group2': '3', 'value': 1}]),

    # LIMIT NUMBER OF RETURNED GROUPS, WITH GROUP "OTHER"
    ({"aggregation": "count",
      "groupBy": "group1", "orderBy": "-value",
      "limit": 1, "showOther": 1},
     [{"group1": "3", "value": 3},
      {"group1": "Other", "value": 3}]),

    # NOT SHOW EMPTY GROUP "OTHER"
    ({"aggregation": "count",
      "groupBy": "group1", "orderBy": "-value",
      "limit": 3, "showOther": 1},
     [{"group1": "3", "value": 3},
      {"group1": "1", "value": 2},
      {"group1": "2", "value": 1}]),

    # GROUP BY PERCENT AGGREGATION
    ({"aggregation": "percent",
      "additionalFilter": json.dumps({"type": "operator",
                                      "data": {"attribute": "group2",
                                               "operator": "=",
                                               "value": "2"}}),
      "groupBy": "group1", "orderBy": "-value"},
     [{"group1": "1", "numerator": 1, "denominator": 2, "value": 0.5},
      {"group1": "3", "numerator": 1, "denominator": 3, "value": 1 / 3},
      {"group1": "2", "numerator": 0, "denominator": 1, "value": 0}]),

    # SORT BY DATE FIELD
    ({"aggregation": "count",
      "truncateDate": "date=day",
      "groupBy": "date__trunc__day", "orderBy": "date__trunc__day"},
     [{"date__trunc__day": date(2020, 10, 1), "value": 2},
      {"date__trunc__day": date(2020, 10, 31), "value": 1},
      {"date__trunc__day": date(2020, 11, 1), "value": 2},
      {"date__trunc__day": date(2020, 11, 2), "value": 1}]),

    # AGGREGATION ON EMPTY QUERY
    ({"search": "4", "aggregation": "count", "groupBy": "group1"},
     []),

    # AGGREGATION ON EMPTY QUERY WITH SORT BY TOTAL GROUP VALUE
    ({"search": "4", "columnIndex": "group1",
      "aggregation": "count",
      "groupBy": "group1,group2", "orderBy": "-group1__index,-group2"},
     []),
]



from django.db import models
from rest_framework.exceptions import ValidationError
from drf_complex_filter.utils import ComplexFilter

from .aggregates import CountIf
from .aggregates import Percentile
from .enums import Aggregation
from .utils import Aggregator


def get_aggregation(
    queryset: models.QuerySet,
    aggregation: str,
    aggregation_field: str = None,
    percentile: str = None,
    additional_filter: str = None,
    group_by: list = None,
    order_by: list = None,
    limit: int = 0,
    limit_by: str = None,
    limit_show_other: bool = False,
    limit_other_label: str = None,
):
    aggregator = Aggregator(queryset=queryset)
    annotations = get_annotations(
        aggregation=aggregation,
        aggregation_field=aggregation_field,
        percentile=percentile,
        additional_filter=additional_filter,
        queryset=queryset,
    )
    return aggregator.get_database_aggregation(
        annotations=annotations,
        group_by=group_by,
        order_by=order_by,
        limit=limit,
        limit_by=limit_by,
        limit_show_other=limit_show_other,
        limit_other_label=limit_other_label,
    )


def get_annotations(
    aggregation: str,
    aggregation_field: str = None,
    percentile: str = None,
    queryset: models.QuerySet = None,
    additional_filter: str = None,
) -> dict:
    if aggregation == Aggregation.COUNT:
        return {"value": models.Count('id')}

    if aggregation == Aggregation.PERCENT:
        if not additional_filter:
            raise ValidationError({"error": "'additionalFilter' is required for 'aggregation=percent'"}, code=422)

        complex_filter = ComplexFilter()
        additional_query = complex_filter.generate_from_string(additional_filter)
        if not additional_query:
            raise ValidationError({"error": "Additional filter cannot be empty"}, code=422)

        return {
            "numerator": CountIf(additional_query),
            "denominator": models.Count("id"),
            "value": models.ExpressionWrapper(
                models.F("numerator") * 1.0 / models.F("denominator"),
                output_field=models.FloatField())
        }

    if not aggregation_field:
        raise ValidationError({"error": f"'aggregationField' is required for 'aggregation={aggregation}'"}, code=422)

    if aggregation == Aggregation.SUM:
        return {"value": models.Sum(aggregation_field)}

    if aggregation == Aggregation.AVERAGE:
        return {"value": models.Avg(aggregation_field)}

    if aggregation == Aggregation.MIN:
        return {"value": models.Min(aggregation_field)}

    if aggregation == Aggregation.MAX:
        return {"value": models.Max(aggregation_field)}

    if aggregation == Aggregation.PERCENTILE:
        if not percentile:
            raise ValidationError({"error": "'percentile' is required for 'aggregation=percentile'"}, code=422)

        model: models.Model = queryset.model
        field = None
        for field_name in aggregation_field.split("__"):
            field = getattr(field, field_name) if field else model._meta.get_field(field_name)

        if field.get_internal_type() != "FloatField":
            return {"value": Percentile(aggregation_field, percentile,
                                        output_field=models.FloatField())}
        return {"value": Percentile(aggregation_field, percentile)}

    raise ValidationError({"error": "Unknown value for param 'aggregation'"}, code=422)



SECRET_KEY = 'secret'

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'rest_framework',
    'tests'
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test',
    }
}

ROOT_URLCONF = "tests.urls"



# Generated by Django 3.1.3 on 2020-11-19 12:54

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestCaseModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('group1', models.CharField(max_length=10)),
                ('group2', models.CharField(max_length=10)),
                ('integer', models.IntegerField()),
                ('float', models.FloatField()),
                ('date', models.DateField()),
                ('datetime', models.DateTimeField()),
                ('duration', models.DurationField()),
            ],
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _


class Aggregation(models.TextChoices):
    AVERAGE = "average", _("average")
    COUNT = "count", _("count")
    MAX = "maximum", _("maximum")
    MIN = "minimum", _("minimum")
    PERCENT = "percent", _("percent")
    PERCENTILE = "percentile", _("percentile")
    SUM = "sum", _("sum")



from typing import Union
from django.db import models
from rest_framework.exceptions import ValidationError


class Aggregator:
    DEFAULT_OTHER_GROUP_NAME = "Other"

    def __init__(self, queryset: models.QuerySet):
        """Get Aggregator instance

        :param queryset: Django Queryset for aggregation
        """
        self.queryset = queryset

    def get_database_aggregation(
            self,
            annotations: dict,
            group_by: list = None,
            order_by: list = None,
            limit: int = None,
            limit_by: str = None,
            limit_show_other: bool = False,
            limit_other_label: str = None
    ) -> Union[dict, list]:
        """Get the aggregation result

        :param annotations: Django aggregation annotation
        :param group_by: list of fields to group the result
        :param order_by: list of fields to sort the result
        :param limit: number of groups to return
        :param limit_by: on which field the limit is set
            default is first field for grouping
        :param limit_show_other: if a limit is set,
            combine other records into an additional group
        :param limit_other_label: title of group "Other"
        :return: list of aggregation results for each set of groups
            in the presence of groupings
            without groupings - a dictionary of the form {value: result}
        """
        if not group_by:
            return self.get_simple_aggregation(annotations)

        queryset = self.queryset.all()
        top_groups_filter = None
        if limit:
            if not limit_by:
                limit_by = group_by[0]
            if not order_by:
                raise ValidationError({"error": "'order_by' is required when limit is set"})

            top_groups_filter = self._get_top_groups_filter(
                field_name=limit_by,
                annotations=annotations,
                order_by=order_by,
                limit=limit)
            queryset = queryset.filter(top_groups_filter)

        if not queryset.exists():
            return []

        queryset = queryset.values(*group_by)
        queryset = queryset.annotate(**annotations)
        if order_by:
            queryset = queryset.order_by(*order_by)

        if not limit or not limit_show_other:
            return list(queryset)

        aggregation_without_top = self.get_aggregation_without_top(
            annotations=annotations,
            group_by=group_by,
            order_by=order_by,
            top_groups_filter=top_groups_filter,
            limit_by=limit_by)
        if not aggregation_without_top:
            return list(queryset)

        aggregation = self._merge_aggregations(
            aggregation_1=list(queryset),
            aggregation_2=aggregation_without_top,
            field_name=limit_by,
            other_group_name=limit_other_label)

        return list(aggregation)

    def get_simple_aggregation(self, annotations) -> dict:
        """Get aggregation without grouping

        :param annotations: Django aggregation annotation
        :return: Dict {value: result}
        """
        aggregation = self.queryset.annotate(
            _group=models.Value(1, output_field=models.IntegerField()))
        aggregation = aggregation.values("_group")
        aggregation = aggregation.annotate(**annotations)
        aggregation = aggregation.values(*annotations.keys())
        return dict(aggregation[0])

    def get_aggregation_without_top(
            self,
            annotations: dict,
            group_by: list,
            order_by: list,
            top_groups_filter: models.Q,
            limit_by: str = None,
    ):
        queryset = self.queryset.exclude(top_groups_filter)
        if queryset.count() == 0:
            return None

        additional_group_by = group_by.copy()
        if limit_by in additional_group_by:
            additional_group_by.remove(limit_by)

        aggregator = Aggregator(queryset=queryset)
        aggregation = aggregator.get_database_aggregation(
            annotations=annotations,
            group_by=additional_group_by,
            order_by=order_by)

        return aggregation

    def _get_top_groups_filter(self,
                               field_name: str,
                               annotations,
                               order_by: list,
                               limit: int) -> models.Q:
        queryset = self.queryset.values(field_name)
        queryset = queryset.annotate(**annotations)
        queryset = queryset.order_by(*order_by)
        top_groups = [group[field_name] for group in queryset[:limit]]

        return models.Q(**{"{}__in".format(field_name): top_groups})

    def _merge_aggregations(
            self,
            aggregation_1: list,
            aggregation_2: Union[dict, list],
            field_name: str,
            other_group_name: str = None
    ) -> list:
        if not other_group_name:
            other_group_name = self.DEFAULT_OTHER_GROUP_NAME

        merged_aggregation = list(aggregation_1)
        if isinstance(aggregation_2, dict):
            aggregation_2[field_name] = other_group_name
            merged_aggregation.append(aggregation_2)
        elif isinstance(aggregation_2, list):
            for result in aggregation_2:
                result[field_name] = other_group_name
                merged_aggregation.append(result)

        return merged_aggregation



from drf_aggregation.routers import AggregationRouter

from .views import TestCaseViewSet


aggregation_router = AggregationRouter()
aggregation_router.register("test", TestCaseViewSet)

urlpatterns = aggregation_router.urls



from drf_aggregation.filters import ColumnIndexFilter
from drf_aggregation.filters import TruncateDateFilter
from drf_aggregation.viewsets import AggregationViewSet
from rest_framework.filters import SearchFilter

from .models import TestCaseModel


class TestCaseViewSet(AggregationViewSet):
    queryset = TestCaseModel.objects.all()
    filter_backends = [SearchFilter, TruncateDateFilter, ColumnIndexFilter]
    search_fields = ["group1"]



from django.db.models.functions import Trunc
from rest_framework.filters import BaseFilterBackend


class TruncateDateFilter(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        truncate_date = request.query_params.get("truncateDate", None)
        if not truncate_date:
            return queryset

        annotations = {}
        for truncate_rule in truncate_date.split(","):
            (field, kind) = truncate_rule.split('=')
            annotations[f'{field}__trunc__{kind}'] = Trunc(field.replace(".", "__"), kind)

        queryset = queryset.annotate(**annotations)

        return queryset



from rest_framework.exceptions import ValidationError
from rest_framework.response import Response

from .helpers import get_aggregation


class AggregationMixin:
    def aggregation(self, request):
        params = request.query_params

        aggregation = params.get("aggregation", None)
        if not aggregation:
            raise ValidationError({"error": "'aggregation' is required"})

        aggregation_field = params["aggregationField"].replace(".", "__") if "aggregationField" in params else None
        limit_by = params["limitBy"].replace(".", "__") if "limitBy" in params else None

        result = get_aggregation(
            queryset=self.filter_queryset(self.get_queryset()),
            aggregation=aggregation,
            aggregation_field=aggregation_field,
            percentile=params.get("percentile", None),
            additional_filter=params.get("additionalFilter", None),
            group_by=self._get_group_by(request),
            order_by=self._get_order_by(request),
            limit=int(params.get("limit", 0)),
            limit_by=limit_by,
            limit_show_other=params.get("showOther", None) == "1",
            limit_other_label=params.get("otherGroupName", None)
        )

        return Response(result)

    @staticmethod
    def _get_group_by(request) -> list:
        group_by = request.query_params.get("groupBy", None)
        group_by = group_by.split(",") if group_by else []

        return [field.replace(".", "__") for field in group_by]

    @staticmethod
    def _get_order_by(request) -> list:
        order_by = request.query_params.get("orderBy", None)
        order_by = order_by.split(",") if order_by else []

        return [field.replace(".", "__") for field in order_by]



from django.db import models


class TestCaseModel(models.Model):
    group1 = models.CharField(max_length=10)
    group2 = models.CharField(max_length=10)
    integer = models.IntegerField()
    float = models.FloatField()
    date = models.DateField()
    datetime = models.DateTimeField()
    duration = models.DurationField()
